#Embedded file name: /Users/versonator/Jenkins/live/output/mac_64_static/Release/python-bundle/MIDI Remote Scripts/Launchpad/SpecialMixerComponent.py
import Live
from _Framework.MixerComponent import MixerComponent
from DefChannelStripComponent import DefChannelStripComponent
from _Framework.ButtonElement import ButtonElement
log_enabled = False
class SpecialMixerComponent(MixerComponent):
    """ Class encompassing several defaultable channel strips to form a mixer """

    def __init__(self, num_tracks, num_returns = 0):
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - Init Start num_tracks=" + str(num_tracks))
        MixerComponent.__init__(self, num_tracks, num_returns)
        self._unarm_all_button = None
        self._unsolo_all_button = None
        self._unmute_all_button = None
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - Init End")
            
    def disconnect(self):
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - disconnect Start")        
        if self._unarm_all_button != None:
            self._unarm_all_button.remove_value_listener(self._unarm_all_value)
            self._unarm_all_button = None
        if self._unsolo_all_button != None:
            self._unsolo_all_button.remove_value_listener(self._unsolo_all_value)
            self._unsolo_all_button = None
        if self._unmute_all_button != None:
            self._unmute_all_button.remove_value_listener(self._unmute_all_value)
            self._unmute_all_button = None
        MixerComponent.disconnect(self)
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - disconnect End")          

    def set_global_buttons(self, unarm_all, unsolo_all, unmute_all):
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - set_global_buttons Start unarm_all=" + str(unarm_all)+ ", unsolo_all=" + str(unsolo_all)+ ", unmute_all=" + str(unmute_all))  
        assert isinstance(unarm_all, (ButtonElement, type(None)))
        assert isinstance(unsolo_all, (ButtonElement, type(None)))
        assert isinstance(unmute_all, (ButtonElement, type(None)))
        
        if self._unarm_all_button != None:
            self._unarm_all_button.remove_value_listener(self._unarm_all_value)
        self._unarm_all_button = unarm_all
        
        if self._unarm_all_button != None:
            self._unarm_all_button.add_value_listener(self._unarm_all_value)
            self._unarm_all_button.turn_off()
        
        if self._unsolo_all_button != None:
            self._unsolo_all_button.remove_value_listener(self._unsolo_all_value)
        self._unsolo_all_button = unsolo_all
        
        if self._unsolo_all_button != None:
            self._unsolo_all_button.add_value_listener(self._unsolo_all_value)
            self._unsolo_all_button.turn_off()
        
        if self._unmute_all_button != None:
            self._unmute_all_button.remove_value_listener(self._unmute_all_value)
        self._unmute_all_button = unmute_all
        
        if self._unmute_all_button != None:
            self._unmute_all_button.add_value_listener(self._unmute_all_value)
            self._unmute_all_button.turn_off()
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - set_global_buttons End")   

    def _create_strip(self):
        return DefChannelStripComponent()

    def _unarm_all_value(self, value):
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - _unarm_all_value Start value=" + str(value))  
        assert self.is_enabled()
        assert (self._unarm_all_button != None)
        assert (value in range(128))
        if value != 0 or not self._unarm_all_button.is_momentary():
            for track in self.song().tracks:
                if (track.can_be_armed and track.arm):
                    track.arm = False
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - _unarm_all_value End")   

    def _unsolo_all_value(self, value):
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - _unsolo_all_value Start value=" + str(value))
        assert self.is_enabled()
        assert (self._unsolo_all_button != None)
        assert (value in range(128))
        if ((value != 0) or (not self._unsolo_all_button.is_momentary())):
            for track in tuple(self.song().tracks) + tuple(self.song().return_tracks):
                if track.solo:
                    track.solo = False
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - _unsolo_all_value End")
            
    def _unmute_all_value(self, value):
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - _unmute_all_value Start value=" + str(value))        
        assert self.is_enabled()
        assert (self._unmute_all_button != None)
        assert (value in range(128))
        if value != 0 or not self._unmute_all_button.is_momentary():
            for track in tuple(self.song().tracks) + tuple(self.song().return_tracks):
                if track.mute:
                    track.mute = False
        if(log_enabled):
            Live.Base.log("SpecialMixerComponent - _unmute_all_value End")                    